
import os
import sys
import argparse
import subprocess
from pathlib import Path
from termcolor import colored
import flurryflake
from flurryflake import FlurryBank

class Host():
    def __init__(self, attacks, benigns, filter):
        self.attack_menu = attacks
        self.benign_menu = benigns
        self.actions = []
        self.filter = filter
        self.bank = FlurryBank(filter)
    #   self.provenance_levels = levels

    #def print_provenance_levels():
    #    for key in self.provenance_levels.keys():
    #        print(colored(str(str(key) + '--' + self.provenance_levels[key]), 'magenta'))

    def print_attacks(self):
        for key in self.attack_menu.keys():
            print(colored(str(key + '--' + self.attack_menu[key]), 'red'))

    def print_benigns(self):
        for key in self.benign_menu.keys():
            print(colored(str(key + '--' + self.benign_menu[key]), 'green'))

    def run(self, scripts, num_loops):
        i = 0
        while i < num_loops:
            '''
                Runs a list of scripts and gathers provenance on each one
            '''
            graph = self.bank.make_flake(self.actions)
            self.bank.connect_client(graph)
            for script in scripts:
                try:
                    print("running " + str(script))
                    fileRead = open(script).read()
                    exec(fileRead)
                except FileNotFoundError as e:
                    print(e)
                    print("Can't open " + str(script))
            self.bank.disconnect_client(graph)
            i += 1

    def save(self, action_str, term_customs, num_loops):
        fn = input("Enter filename to save this configuration (cwd = " + os.getcwd() + "; leave blank to not save config): ")
        if (fn != ""):
            f = open(fn, "w")
            f.write(action_str + "\n")
            for tc in term_customs:
                f.write(tc)
            f.write(str(num_loops) + "\n")
            f.close()

def read_input_file(f):
    cfg_txt = ""
    if f is not None:
        cfg_path = Path(os.getcwd() + "/" + f)
        if not cfg_path.is_file():
            print("Could not find the automation file; try again.")
            sys.exit()
        cfg_txt = open(cfg_path).read()
    return cfg_txt

#def get_level(cfg_txt, cfg_lines, cfg_custom_count):
#    prov_level = 1
#    if cfg_txt == "":
#        prov_level = int(input("Select provenance capture granularity: "))
#        while prov_level < 1 or prov_level > 4:
#            prov_level = int(input("Invalid option selected. Try again."))
#    else:
#        prov_level = int(cfg_lines[2 + cfg_custom_count])
#        print(prov_level)
#    return prov_level

def get_loops(cfg_txt, cfg_lines, cfg_custom_count):
    num_loops = 1
    if cfg_txt == "":
        num_loops = int(input("Execution loop is constructed. How many iterations would you like to make? "))
    else:
        num_loops = int(cfg_lines[1 + cfg_custom_count])
        print(num_loops)
    return num_loops



import os
import warnings
from pathlib import Path

import flurryflake
from . import host

warnings.filterwarnings("ignore", category=DeprecationWarning)

#config options
SAVE_TO_DISK = True

## Start menu coding
attack_menu_options = {
    'customattack': 'Use My Own Attack'
}

benign_menu_options = {
    'custombehavior': 'Use My Own Custom Behavior'
}

class FlurrySDN(host.Host):
    def __init__(self, filter, cfg_path=None):
        host.Host.__init__(self, attack_menu_options, benign_menu_options, filter)
        cfg_txt = host.read_input_file(cfg_path)
        cfg_lines = cfg_txt.split("\n")
        cfg_custom_count = 0

        try:
            if cfg_txt == "":
                self.print_attacks()
                self.print_benigns()
                action_str = input('Select one or more attacks to run in a comma-separated list: ')
            else:
                action_str = cfg_lines[0]
                print(action_str)
            self.actions = action_str.split(",")
            scripts = []
            term_customs = [] # For writing config files
            for action in self.actions:
                script = Path("")
                if action == "customattack" or action == "custombehavior":
                    if cfg_txt == "":
                        local_path = ""
                        while (not script.is_file()):
                            local_path = input('Provide the local path to your custom script: ')
                            script = Path(os.getcwd() + "/" + local_path)
                            if (not script.is_file()):
                                print('Invalid file path provided. Try again.')
                        term_customs.append(local_path)
                        #action = str(input("Provide a single word identifier for this custom execution (i.e. \'bruteforce\')")).split(" ")[0]
                    else:
                        cfg_custom_count += 1
                        script = Path(os.getcwd() + "/" + cfg_lines[cfg_custom_count])
                else:
                    print('Invalid option ' + action + ' provided.')
                if script.is_file():
                    scripts.append(script)
        except Exception as e:
            print(e)

        # Other options
        num_loops = host.get_loops(cfg_txt, cfg_lines, cfg_custom_count)
        self.run(scripts, num_loops)
        if cfg_txt == "":
            self.save(action_str, term_customs, num_loops)
